Strict
Framework brl.blitz
Import brl.max2d
Import brl.timer
Import brl.pngloader
?Win32
Import brl.d3d9max2d
?Not Win32
Import brl.glmax2d
?

Graphics 800,600

Global ORDER = 20 ' Ordnung



If AppArgs.length>1 And AppArgs[1] Then ORDER = Int(AppArgs[1])

Global info:String

AutoMidHandle True
Local pix:TPixmap = CreateCurve(ORDER)
Local img:TImage
Local px = GraphicsWidth()/2, py = GraphicsHeight()/2
Local mpx = px, mpy = py
Local mxs, mys, mzs, size# = 1
Local timer:TTimer = CreateTimer(50)

img = LoadImage(pix)

Repeat
	Cls
	
	If KeyHit(KEY_RETURN) Then
		SavePixmapPNG pix, "curve.png",3
		addinfo "Saved."
	EndIf
	
	mxs = MouseXSpeed()
	mys = MouseYSpeed()
	mzs = MouseZSpeed()

	If MouseDown(MOUSE_MIDDLE) Then SetScale 1, 1; size = 1; px = mpx; py = mpy
	If MouseDown(MOUSE_LEFT) Then px:+mxs ; py:+mys
	If mzs Then size:+mzs/30.0
	
	SetScale size, size
	DrawImage img, px, py
	drawinfo

	Flip 0
	WaitTimer timer
Until KeyHit(KEY_ESCAPE) Or AppTerminate()
End

Function drawinfo()
	Local s:String = info, y = -6
	SetScale 1,1	
	Repeat
		y:+16
		s = s[s.find("~n")+1..]
		DrawText s[..s.find("~n")], 10, y
		If s.find("~n")<0 Then Exit
	Forever
	DrawText s, 10, y
EndFunction

Function addinfo(s:String)
	info:+"~n"+s
	drawinfo
EndFunction

Function CreateCurve:TPixmap(o)
	Local t:TTurtle = New TTurtle
	Local x, y, w, h, time

	time = MilliSecs()
	Cls
	addinfo "Creating Curve String ..."
	Flip
	t.drawing = CreateCurveString(o)

	Cls
	addinfo "Analyzing Curve ..."
	Flip
	AnalyzeCurve t.drawing, x, y, w, h

	Cls
	addinfo "Width: " + w + "      Height: " + h
	addinfo "Drawing Curve ...  0%"
	Flip
	t.x = x
	t.y = y
	DebugLog "W: "+ w
	DebugLog "H: "+ h
	t.pixmap = CreatePixmap(w, h, PF_RGB888)
	
	
	t.drawtopixmap
	time = MilliSecs() - time
	
	addinfo "Finished. Time needed : "+time+ " ms"
	addinfo "Press Return to save."
	Return t.pixmap
EndFunction

Function curve:String(o)
	Local s:String, g:String
	If o = 1 Then
		Return "R"
	Else
		g = curve(o-1)
		s = g + "R" + g[..g.length/2] + "L" + g[g.length/2+1..]
		Return s
	EndIf
EndFunction

Function CreateCurveString:String(o)
	Return curve(o)
EndFunction

Function AnalyzeCurve(c:String, xc Var, yc Var, width Var, height Var)
	Local x, y, i, w = 270
	Local maxx, maxy
	Local minx, miny
	For i = 1 Until c.length-1 Step 2
		Select c[i-1]
		Case Asc "R"
			w:+90
		Case Asc "L"
			w:-90
		EndSelect
		Select w Mod 360
		Case 0
			x:+1
		Case 90
			y:+1
		Case 180
			x:-1
		Case 270
			y:-1
		EndSelect
		Select c[i]
		Case Asc "R"
			w:+90
		Case Asc "L"
			w:-90
		EndSelect			
		Select w Mod 360
		Case 0
			x:+1
		Case 90
			y:+1
		Case 180
			x:-1
		Case 270
			y:-1
		EndSelect
		If x > maxx Then maxx = x
		If x < minx Then minx = x
		If y > maxy Then maxy = y
		If y < miny Then miny = y
	Next
	xc = -minx+1
	yc = -miny+1
	maxx :- minx
	maxy :- miny
	width = maxx + maxx Mod 2 + 2
	height = maxy + maxy Mod 2 + 2
EndFunction

Type TTurtle
	Field x#,y#
	Field w
	Field pixmap:TPixmap

	Field drawing:String	
	
	Method New()
		w = 270
		x = GraphicsWidth()/2
		y = GraphicsHeight()/2	
	EndMethod
	
	Method drawtopixmap()
		Global r#=256, g#=256, b#=256, rgb
		Local x1, y1
		Local i, e
		Local width = PixmapWidth(pixmap)
		Local height = PixmapHeight(pixmap)
		Local p1, p2	
		ClearPixels pixmap, $010101
		
		For i = 1 Until drawing.length-1 Step 2
			x1 = x ; y1 = y 	
			
			p1 = 100.0*i/drawing.length
			If p1 > p2 Then
				info = info[..info.length-3]
				If p1<10 Then info:+" "
				info :+ String(p1)+"%"
				p2 = p1
				Cls
				drawinfo
				Flip
			EndIf	
			
			r = r + w/144000.0
			b = 128 + Sin(e*36)*127
			g = g + 0.005
			While r > 255 ; r:-156 ; Wend
			While g > 255 ; g:-176 ; Wend
			While b > 255 ; b:-196 ; Wend
			rgb = (Int(r) Shl 16) | (Int(g) Shl 8) | Int(b)			
				
			
			Select drawing[i-1]
			Case Asc "R"
				w:+90
			Case Asc "L"
				w:-90
			EndSelect
			Select w Mod 360
			Case 0
				If y > 0 Then WritePixel pixmap, x, y-1, rgb
				x:+1
			Case 90
				If x < width-1 Then WritePixel pixmap, x+1, y, rgb 
				y:+1
			Case 180
				If y < height-1 Then WritePixel pixmap, x, y+1, rgb
				x:-1
			Case 270
				If x > 0 Then WritePixel pixmap, x-1, y, rgb
				y:-1
			EndSelect
			
			Select drawing[i]
			Case Asc "R"
				w:+90
			Case Asc "L"
				w:-90
			EndSelect			
			Select w Mod 360
			Case 0
				x:+1
			Case 90
				y:+1
			Case 180
				x:-1
			Case 270
				y:-1
			EndSelect
			WritePixel pixmap, x, y, rgb
			WritePixel pixmap, x1, y1, rgb

		Next
		info = info[..info.length-3]+"100%"
		Cls	
		drawinfo
		Flip
	EndMethod
EndType
